#!/usr/bin/env python
# Copyright 2017 Marco Galardini and John Lees

'''Count unique patterns'''

mem_adjust = 10


def get_options():
    import argparse

    description = 'Calculate p-value threshold using Bonferroni correction'
    parser = argparse.ArgumentParser(description=description)

    parser.add_argument('patterns',
                        help='File of patterns from pyseer')

    parser.add_argument('--alpha',
                        default=0.05,
                        type=float,
                        help='Family-wise error rate')
    parser.add_argument('--cores',
                        default=1,
                        help='Number of cores to use')
    parser.add_argument('--memory',
                        default=1024,
                        help='Maximum memory to use (in Mb)')
    parser.add_argument('--temp',
                        default='/tmp',
                        help='Directory to write tmp files to')

    return parser.parse_args()


if __name__ == "__main__":
    options = get_options()

    import sys
    import subprocess
    from decimal import Decimal

    command = "LC_ALL=C sort -u "
    if options.cores > 1:
        command +=  "--parallel=" + str(options.cores)
    command += (" -S " + str(int(options.memory) - mem_adjust) + "M" +
               " -T " + options.temp +
               " " + options.patterns +
               " | wc -l")

    p = subprocess.check_output(command, shell=True, universal_newlines=True)
    print("Patterns:\t" + p.rstrip())
    print("Threshold:\t" + '%.2E' % Decimal(options.alpha/float(p.rstrip())))


import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
from sklearn.decomposition import PCA
from sklearn.preprocessing import StandardScaler
from scipy.spatial.distance import pdist, squareform
from skbio.stats.distance import permanova
from scipy.cluster.hierarchy import linkage, fcluster, cut_tree
from skbio import DistanceMatrix
import random
from sklearn.cluster import DBSCAN
from sklearn.manifold import MDS
from sklearn.manifold import TSNE
import os
import umap

def cluster_samples(mash_file, output_folder):
    # Load data
    mash = pd.read_csv(mash_file, sep="\t", index_col=0)
    # Update row and column names
    mash.index = [os.path.splitext(os.path.basename(row))[0] for row in mash.index]
    mash.columns = [os.path.splitext(os.path.basename(col))[0] for col in mash.columns]

    # Convert condensed distance matrix to a square distance matrix
    square_dist = squareform(mash)

    # Cluster using hierarchical clustering
    linkage_matrix = linkage(square_dist, method='ward')
    plt.title('Hierarchical clustering')

    # Perform clustering with the best number of clusters
    max_d = 0.44 * np.max(linkage_matrix[:, 2])
    clusters = cut_tree(linkage_matrix, height=max_d)
    cluster_labels = [str(i + 1) for i in clusters.flatten()]

    # Plot clusters
    cluster_labels = [str(label).strip('[]') for label in clusters]  # Remove "[" and "]" characters
    phylogroup = pd.DataFrame({'Phylogroup': cluster_labels, 'Sample': mash.index})
    phylogroup.to_csv(f"{output_folder}/phylogroups.txt", sep="\t", index=False)
    
    phylogroup_wide = phylogroup.copy()
    # Convert the 'Phylogroup' column into dummy variables
    phylogroup_dummies = pd.get_dummies(phylogroup_wide['Phylogroup'])

    # Concatenate the 'Sample' column with the dummy variables
    phylogroup_wide = pd.concat([phylogroup_wide['Sample'], phylogroup_dummies], axis=1)

    # Set 'Sample' column as the DataFrame index
    phylogroup_wide.set_index('Sample', inplace=True)

    # Replace the dummy variable values (0 and 1) with the actual phylogroup names
    phylogroup_wide = phylogroup_wide.apply(lambda x: x.astype(str).str.replace('0', '').str.replace('1', x.name))
    
    # Replace all numbers with 1
    phylogroup_wide = phylogroup_wide.replace(np.nan, 0).replace('[0-9]+', 1, regex=True)

    # Fill empty spaces with 0s
    phylogroup_wide = phylogroup_wide.mask(phylogroup_wide != 1, other=0)
    phylogroup_wide = phylogroup_wide.fillna(0)
    
    phylogroup_wide.to_csv(f"{output_folder}/phylogroup_wide.txt", sep="\t", index=True)


    # Load the color information from the phylogroups
    # Generate a random color for each unique phylogroup
    unique_phylogroups = phylogroup['Phylogroup'].unique()
    random.shuffle(unique_phylogroups)
    phylogroup_colors = {group: '#' + ''.join(random.choices('0123456789ABCDEF', k=6)) for group in unique_phylogroups}
   # phylogroup_colors = {'1': "#89B4AE", '0':"#B3A3CA"}
   # phylogroup_colors = {'3' : "#97C079", '6': "#465AA8", '10':"#BC1F75", '8': "#DFAE41", '9': "#8773B4"}

    # Map the phylogroup colors to each sample in the dataframe
    row_colors = phylogroup.set_index('Sample')['Phylogroup'].map(phylogroup_colors)

   # Plot clusters using PCA
    pca = PCA(n_components=2)
    pca.fit(mash)
    pca_scores = pca.transform(mash)
    pca_df = pd.DataFrame(pca_scores, columns=['PC1', 'PC2'])
    pca_df['Cluster'] = cluster_labels

    plt.figure()
    sns.scatterplot(x='PC1', y='PC2', hue='Cluster', palette=phylogroup_colors, data=pca_df)  # Add row_colors
    plt.title('Clusters of samples by PCA')
    plt.xlabel(f'PC1 ({pca.explained_variance_ratio_[0]*100:.2f}%)')  # Add explained variance ratio to x-axis label
    plt.ylabel(f'PC2 ({pca.explained_variance_ratio_[1]*100:.2f}%)')  # Add explained variance ratio to y-axis label
    # Move the legend to the right of the graph
    plt.legend(title='Cluster', loc='center left', bbox_to_anchor=(1, 0.5))
    plt.tight_layout()

    # Save the plot to a PNG file
    plt.savefig(f"{output_folder}/mash_PCA.png", dpi=300, bbox_inches='tight')

    # Perform t-SNE on the distance matrix
    tsne = TSNE(n_components=2)
    embedding = tsne.fit_transform(mash)

    # Create a DataFrame with the t-SNE coordinates and cluster labels
    tsne_df = pd.DataFrame(embedding, columns=['TSNE1', 'TSNE2'])
    tsne_df['Cluster'] = cluster_labels

    # Plot the clusters using t-SNE
    plt.figure()
    sns.scatterplot(x='TSNE1', y='TSNE2', hue='Cluster', palette=phylogroup_colors, data=tsne_df, legend=False)
    plt.title('Clusters of samples by t-SNE')
    plt.xlabel('TSNE1')
    plt.ylabel('TSNE2')
    plt.legend(bbox_to_anchor=(1.02, 1), loc='upper left')
    plt.savefig(f"{output_folder}/mash_tSNE.png", dpi=300, bbox_inches='tight')

    # Plot heatmap of clusters with color annotations and inverted colormap
    clustermap = sns.clustermap(mash, cmap='binary', row_cluster=True,  row_colors=row_colors, row_linkage=linkage_matrix, col_linkage=linkage_matrix)

    # Adjust the size of the legend
    cbar = clustermap.cax.get_position()
    clustermap.cax.set_position([cbar.x0 + 0, cbar.y0 + 0.1, cbar.width * 0.5, cbar.height * 0.5])  # Make the legend 5 times smaller

    # Remove tick marks from the axis
    clustermap.ax_heatmap.set_xticks([])
    clustermap.ax_heatmap.set_yticks([])

    # Remove axis names
    clustermap.ax_heatmap.set_xlabel('')
    clustermap.ax_heatmap.set_ylabel('')
    # Save the heatmap to a file
    plt.savefig(f"{output_folder}/mash_heatmap.png", dpi=300, bbox_inches='tight')
    return(phylogroup)



import sys
import os
import pandas as pd

def process_checkm_files(output_prefix: str, input_files: list):
    """
    Process CheckM output files and generate a consolidated statistics file.

    Args:
        output_prefix (str): Prefix for the output file path.
        input_files (list): List of input file paths.

    Returns:
        None
    """
    # Define output path
    output_path = os.path.join(output_prefix, "Quality/CheckM/checkm_stats.txt")

    checkm = pd.DataFrame()

    # Read and process input files
    for input_file in input_files:
        line = pd.read_csv(input_file, sep='\t', header=None)
        checkm = pd.concat([checkm, line], ignore_index=True)

    # Process the checkm2 column
    checkm2 = checkm.drop(checkm.columns[0], axis=1)
    checkm2 = checkm2.astype(str).applymap(lambda x: str(x).replace("'", "").replace("{", "").replace("}", ""))
    checkm2 = checkm2[1].str.split(',', expand=True)
    checkm2 = checkm2.applymap(lambda x: x.split(":")[1])
    checkm2.columns = ["GC", "GC std", "Genome size", "ambiguous bases", "scaffolds", "contigs",
                       "Longest scaffold", "Longest contig", "N50 scaffolds", "N50 contigs", "Mean scaffold length",
                       "Mean contig length", "Coding density", "Translation table", "predicted genes"]

    # Process the Sample column
    checkm2.insert(0, "Sample", checkm[0])

    # Save the DataFrame to a tab-separated text file
    checkm2.to_csv(output_path, sep='\t', index=False)

# Get input file paths from command-line arguments
input_files = sys.argv[2:]

# Define output path
output_prefix = sys.argv[1]

# Call the function
process_checkm_files(output_prefix, input_files)



# ----------------------------------------------------------------------------
# Copyright (c) 2022--, Panpiper development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------

import sys
import getopt
import os
import numpy as np
import pandas as pd
import argparse
from collections import Counter

def checkm_filter(sample_list, file, completeness, contamination, N50):
    """
    Filter samples based off checkm parameters

    Parameters 
    ----------
    sample_list: string, required
    A text file which will contain names of all samples which pass the filters

    file: string, required
    A string of the filename of the checkm output, the log file

    completeness: int, default = 95
    An integer of the cutoff value at which to remove samples if they are under this number

    contamination: int, default = 5
    An integer of the cutoff value at which to remove samples if they are above this number

    N50: int, default = 5000
    An integer of the max N50 value, samples are filtered out if they exceed this value

    Returns
    ----------
    sample_pass: list
    The samples that passed the filters at this step returned as a list

    sample_fail: dataframe
    The samples that failed the filtes at this step returned 
    """

    # Reads the checkm text file
    df = pd.read_csv(file, sep='\t', header=0)
    # Includes completeness, contamination, and strain heterogeneity
    # Filter dataframe
    # For each row in the dataframe, only include a sample in a list
    # If it is above a certain value in each of the three categories
    # Completion above 95, Contamination below 5
    df_pass = df[df["Completeness"] >= completeness]
    df_pass = df_pass[df_pass["Contamination"] <= contamination]
    df_pass = df_pass[df_pass["Contig_N50"] >= N50]
    # Return the list of samples
    sample_pass = df_pass["Name"].tolist()
    sample_pass = [*set(sample_pass)]

    # Return the failed samples as a dataframe
    sample_fail = df[(df["Completeness"] < completeness) |
                     (df["Contamination"] > contamination) | 
                     (df_pass["Contig_N50"] < N50)]

    return sample_pass, sample_fail

def ani_filter(sample_list, file, ani, reference, output):
    """
    Filter samples based off checkm parameters

    Parameters 
    ----------
    sample_list: string, required
    A text file which will contain names of all samples which pass the filters

    file: string, required
    A string of the filename of the ani output, the matrix file

    ani: int, default = 95
    An integer of the cutoff value at which to remove samples if they are under this number compared to the reference

    reference: string, required
    The name of the reference sample as it appears in the ani matrix file

    Returns
    ----------
    sample_pass: list
    The samples that passed the filters at this step returned as a list

    sample_fail: dataframe
    The samples that failed the filtes at this step returned as a dataframe which contain information about why they failed
    """

    # Reads the ani text file
    df = pd.read_csv(file, sep='\t', header=0, index_col=None)
    df = df.fillna(value=0)
    df.columns = ["sample", "reference", "ani", "contig1", "contig2"]
    df["sample"] = df["sample"].apply(os.path.basename)
    df["reference"] = df["reference"].apply(os.path.basename)
    df["reference"] = df["reference"].str.replace(".fna", "")
    df["sample"] = df["sample"].str.replace(".fna", "")
    df.to_csv(output+'/fastani_reformat.csv', sep='\t')
    # Filter dataframe based off first column, select sample
    # Only add samples to the list if they are above 95% in the third column
    df_pass = df[df["ani"] >= ani]
    sample_fail = df.loc[df["ani"] < ani]
    # Add the samples from the second column to the list
    # Return the list of samples by APPENDING TO LIST
    sample_list = df_pass["sample"].tolist()
    sample_list = [*set(sample_list)]
    return sample_list, sample_fail


if __name__ == "__main__":
    checkm_list, ani_list = [], []
    # Result: 0 checkm,1 ani,2 outfile,3 completeness,4 contamination,
    #   5 strain_heterogeneity,6 ani_cutoff,7 reference
    # The default cut offs are based off the "Reasons an assembly is excluded from RefSeq" at the NCBI webserver
    parser = argparse.ArgumentParser()
    parser.add_argument("-k", "--checkm")
    parser.add_argument("-a", "--ani")
    parser.add_argument("-o", "--outpath")
    parser.add_argument("-p", "--completeness", default=95, type=float)
    parser.add_argument("-c", "--contamination", default=5, type=float)
    parser.add_argument("-ac", "--ani_cuttoff", default=95, type=float)
    parser.add_argument("-r", "--reference")
    parser.add_argument("-n", "--N50", default=5000, type=float)
    args = parser.parse_args()
    print(args)
    if args.outpath:
        # create a file with the names of all the fastas in it
        if args.checkm:
            checkm_list, checkm_fail = checkm_filter(
                checkm_list, args.checkm, args.completeness, args.contamination, args.N50)
        if args.ani:
            ani_list, ani_fail = ani_filter(
                ani_list, args.ani, args.ani_cuttoff, args.reference, args.outpath)
        # Now I want to just keep the duplicates
        checkm_list += ani_list
        # Count the occurrences of each element in the combined list
        element_counts = Counter(checkm_list)
        # Filter the elements that occur exactly 2 times
        checkm_list = [element for element, count in element_counts.items() if count == 2]
        # merge dataframes of failed samples
        checkm_fail = checkm_fail.rename(columns={'Name': 'Sample'})
        if len(args.outpath) > 0:
            passed = args.outpath + "/sample_list.txt"
            with open(passed, mode='wt', encoding='utf-8') as myfile:
                myfile.write('\n'.join(checkm_list))
        checkm_fail.to_csv(args.outpath+'/failed_samples_checkm.csv', sep='\t')
        ani_fail.to_csv(args.outpath+'/failed_samples_ani.csv', sep='\t')


# ----------------------------------------------------------------------------
# Copyright (c) 2022--, Panpiper development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------

import sys
import os
import numpy as np
import pandas as pd


def long_to_wide(file, output, path=""):
    """
    Open the text file and convert format from long to wide (matrix). 
    Writes the output to the file specified.

    Parameters 
    ----------
    file: string, required
        The filename of the file read in 

    output: string, required
        The filename of the desired output

    path: string
        The full path of the file

    """
    # Reads the ani text file
    df = pd.read_csv(file, sep='\t', names=[
                     "Reference", "Sample", "PercentIdentity", "Contig1", "Contig2"])
    # TODO: Change names - remove path
    for row in range(0, len(df.index)):
        df.at[row, 'Reference'] = os.path.basename(df.at[row, 'Reference'])
        df.at[row, 'Sample'] = os.path.basename(df.at[row, 'Sample'])
    df['Reference'] = df['Reference'].str.replace('\\.fasta', '')
    df['Reference'] = df['Reference'].str.replace('\\.fna', '')
    df['Sample'] = df['Sample'].str.replace('\\.fasta', '')
    df['Sample'] = df['Sample'].str.replace('\\.fna', '')
    df = df.pivot_table(index='Reference', columns='Sample',
                        values='PercentIdentity', aggfunc='mean')
    if os.path.exists(output):
        os.remove(output)
        print("File exists, overwriting")
    df.to_csv(output, header=True, index=True, sep='\t', mode='a')

    return 0


if __name__ == "__main__":
    if len(sys.argv) == 3:
        long_to_wide(sys.argv[1], sys.argv[2])
    elif len(sys.argv) == 4:
        long_to_wide(sys.argv[1], sys.argv[2], sys.argv[3])
    else:
        print("Enter file, output name, and optional path to remove from header")



#!/usr/bin/env python
# Copyright 2017 Marco Galardini and John Lees

'''Extract a distance matrix from a phylogeny'''


def get_options():
    import argparse

    description = 'Extract a distance matrix from a phylogeny'
    parser = argparse.ArgumentParser(description=description)

    parser.add_argument('phylogeny',
                        help='Tree file')

    parser.add_argument('--format',
                        default="newick",
                        help="Format of tree file [Default: newick]")
    parser.add_argument('--midpoint',
                        action='store_true',
                        default=False,
                        help='Midpoint root the tree before calculating '
                             'distances.')
    method_group = parser.add_mutually_exclusive_group()
    method_group.add_argument('--lmm', '--calc-C',
                              action='store_true',
                              help='Produce var-covar matrix C (as from PDDIST). '
                                   'Always uses branch lengths.')
    method_group.add_argument('--topology',
                              action='store_true',
                              default=False,
                              help='Ignore branch lengths, and only use topological '
                                   'distances')

    return parser.parse_args()

if __name__ == "__main__":
    options = get_options()

    import sys
    import re
    import pandas as pd
    import dendropy

    tree = dendropy.Tree.get(
        path=options.phylogeny,
        schema=options.format,
        preserve_underscores=True)

    if options.midpoint:
        tree.reroot_at_midpoint(update_bipartitions=True, suppress_unifurcations=False)

    d = {}
    pdm = tree.phylogenetic_distance_matrix()
    for idx1, taxon1 in enumerate(tree.taxon_namespace):
        d[taxon1.label] = d.get(taxon1.label, {})
        for taxon2 in tree.taxon_namespace:
            if taxon2.label not in d[taxon1.label].keys():
                if options.lmm:
                    mrca = pdm.mrca(taxon1, taxon2)
                    d[taxon1.label][taxon2.label] = mrca.distance_from_root()
                elif options.topology:
                    d[taxon1.label][taxon2.label] = pdm.path_edge_count(taxon1, taxon2)
                else:
                    d[taxon1.label][taxon2.label] = pdm.patristic_distance(taxon1, taxon2)

    m = pd.DataFrame(d)
    m = m.reindex(m.columns)
    m.to_csv(sys.stdout,
             sep='\t')



import pandas as pd
import numpy as np
import re

def genes_long(genes, clusters, output):
    # Load data
    genes = pd.read_csv(genes, header=0, dtype={'Gene': str}, low_memory=False, comment='#')
    clusters = pd.read_csv(clusters, sep="\t")
    genes = genes.drop(genes.columns[[6, 7, 8, 9, 10]], axis=1)
    genes = genes.iloc[:, list([0, 2, 3]) + list(range(9, genes.shape[1]))]

    # Melt data
    genes_long = pd.melt(genes, id_vars=["Gene", "Annotation", "No. isolates"], var_name="Sample")
    genes_long = pd.merge(genes_long, clusters, on="Sample", how="left")
    genes_long = genes_long.rename(columns={genes_long.columns[4]: "contig"})

    genes_long['contig'] = genes_long['contig'].str.replace(r"[\[\]']", '', regex=True)
    # Split values in the 'contig' column by semicolon (;) and explode the resulting list into separate rows
    genes_long['contig'] = genes_long['contig'].str.split(';')
    genes_long = genes_long.explode('contig')

    # Save data to files
    genes_long.to_csv(output + "/genes_long.txt", sep="\t", index=False, quoting=None)

    # Create temporary dataframe
    genes_long['presence'] = 0
    genes_long.loc[genes_long['contig'].notna(), 'presence'] = 1

    return(genes_long) 

def gene_matrix_phylogroup(genes_long, output):
    # Group and summarize by Gene and Phylogroup
    genes_long = pd.read_csv(genes_long, sep="\t")
    genes_long['presence'] = 0
    genes_long.loc[genes_long['contig'].notna(), 'presence'] = 1

    # Group and summarize by Gene and Phylogroup
    genes_phylo = genes_long.iloc[:, [0, 3, 5, 6]]
    genes_phylo = genes_phylo.drop_duplicates()
    genes_phylo = genes_phylo.iloc[:, [0, 2, 3]]
    genes_phylo = genes_phylo.groupby(["Gene", "Phylogroup"]).agg({"presence": "sum"}).reset_index()


    # Pivot table and calculate percentages
    genes_phylo = genes_phylo.pivot(index='Gene', columns='Phylogroup', values='presence').fillna(0)
    genes_phylo.to_csv(output + "/genes_phylogroup_matrix.txt", sep="\t", index=True, quoting=None)

    genes_phylo_perc = round(genes_phylo.apply(lambda x: (x / max(x)) * 100, axis=0))
    # Rename columns and write to file
    genes_phylo_perc.reset_index(inplace=True)
    genes_phylo_perc.rename(columns={"Gene": "Gene"}, inplace=True)

    genes_phylo_perc.to_csv(output + "/genes_phylogroup_matrix_perc.txt", sep="\t", index=False, quoting=None)

def gene_matrix(genes,output):
    # Extract relevant columns
    genes = pd.read_csv(genes, header=0, dtype={'Gene': str}, low_memory=False, comment='#')
    genes = genes.drop(genes.columns[[6, 7, 8, 9, 10]], axis=1)
    genes_matrix = genes.iloc[:, [0, 9, *range(10, genes.shape[1])]]

    # Set gene name as index and remove gene column from matrix
    genes_matrix.index = genes_matrix['Gene']
    genes_matrix = genes_matrix.iloc[:, 1:]


    genes_matrix.to_csv(output + "/genes_matrix_names.txt", sep="\t", index=True, quoting=None)

    # Replace all NA values with 0
    genes_matrix = genes_matrix.fillna(0)    
    genes_matrix = genes_matrix.where(genes_matrix == 0, other=1)

    genes_matrix.to_csv(output + "/genes_matrix.txt", sep="\t", index=True, quoting=None)

    return(genes_matrix)

def process_data(bakta, genes, eggnog, output):
    # Load data
    bakta = pd.read_csv(bakta, delimiter="\t", header=0, comment='#')
    genes = pd.read_csv(genes, delimiter=",", low_memory=False, header=0, comment='#')
    eggnog = pd.read_csv(eggnog, delimiter="\t", header=0, comment='#')
    # Remove the '#' from the header row
    eggnog.columns = eggnog.columns.str.lstrip('#')

    # Clean data
    genes = genes.iloc[:, :6].join(genes.iloc[:, 11:], how='outer')
    bakta = bakta.rename(columns={bakta.columns[2]:"Gene_Name" , bakta.columns[0] : "Gene"})
    bakta = genes.iloc[:, :8].merge(bakta, on="Gene", how="left")
    bakta.drop_duplicates(subset="Gene", keep="first", inplace=True)
    eggnog = eggnog.rename(columns={eggnog.columns[0]:"Gene"})
    genes_anno = bakta.merge(eggnog, on="Gene", how="left").drop_duplicates(subset="Gene", keep="first")
    genes_anno["group"] = "ACCESSORY"
    genes_anno.loc[genes_anno["No. isolates"] >= (genes_anno["No. isolates"].max() * 0.99), "group"] = "CORE"
    genes_anno.loc[genes_anno["No. isolates"] / genes_anno["No. isolates"].max() <= 0.01, "group"] = "UNIQUE"
    gene_columns = list(genes.columns)

    #Write output
    genes_anno.to_csv(f"{output}/genes_anno.txt", sep='\t', index=False)

    return(genes_anno)


# ----------------------------------------------------------------------------
# Copyright (c) 2022--, Panpiper development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------
from Bio import SeqIO
from Bio.SeqRecord import SeqRecord
import sys
import os


def translate(fasta, output):
    with open(output, 'w') as aa_fa:
        for dna_record in SeqIO.parse(fasta, 'fasta'):
            dna_seqs = [dna_record.seq]
            # generate all translation frames
            aa_seqs = (s[i:].translate(to_stop=True)
                       for i in range(3) for s in dna_seqs)
            # select the longest one
            max_aa = max(aa_seqs, key=len)
            # write new record
            aa_record = SeqRecord(max_aa, id=dna_record.id, description="")
            SeqIO.write(aa_record, aa_fa, 'fasta')


if __name__ == "__main__":
    fasta = sys.argv[1]
    output = sys.argv[2]
    translate(fasta, output)



# ----------------------------------------------------------------------------
# Copyright (c) 2022--, Panpiper development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------

import sys
import os
import numpy as np
import pandas as pd


def agg_df(x):
    return pd.Series(dict(Length=x['Length'].sum(),
                          Contig="%s" % ', '.join(x['Contig'])))


def reformat_kraken(basename, krakenpath):
    """
    Changes the kraken output to be a dataframe 
    Parameters 
    ----------
    basename: string, required
        The name of the sample and basename of the file

    krakenpath: string, required
        The name of out directory, and the directory where the kraken files are stored
    """
    # Read in the file
    filename = krakenpath+"/"+basename+".out"

    df = pd.read_csv(filename, sep='\t', header=None, names=[
                     "D1", "Contig", "ID", "Length", "D2"])

    # Remove unnecessary columns
    df = df[["ID", "Length", "Contig"]]

    # Get the sum of the column, Length
    df_sum = df['Length'].sum()

    # Aggregate by the second column, concatenate all the contig IDs together in a list
    df = df.groupby('ID').agg({'Length': 'sum', 'Contig': ','.join})

    # add a column called percent which is the percent of the sum for each line
    df["Percent"] = df.Length / df_sum * 100

    # add another column which is the base name this column should be called
    df["Sample"] = basename

    # reformat again to : Sample, ID, Percent, Length, Contig
    df = df[['Sample', 'Percent', 'Length', 'Contig']]

    df.to_csv(krakenpath+"/"+basename+"_reformat.txt", header=True, sep='\t')


if __name__ == "__main__":
    basename = sys.argv[1]
    krakenpath = sys.argv[2]
    reformat_kraken(basename, krakenpath)



# ----------------------------------------------------------------------------
# Copyright (c) 2022--, Panpiper development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------

import sys
import getopt
import os
import numpy as np
import pandas as pd
import argparse


def amr_concat(files, output):
    """
    Concatenate the output of AMR into master AMR matrix 

    Parameters 
    ----------
    files: list of strings, required
    A list of input files from the output of AMRFinderPlus

    output: string, required
    A string representing the desired output filename, should include path

    Returns
    ----------
    Writes the matrix to the output file. 

    TODO: Should raise exceptions or write warnings
    """

    df = pd.DataFrame()
    # For each filename in the list, read it in and concat it to the new dataframe, df
    for f in files:
        df_new = pd.read_csv(f, sep="\t")
        df_new['Sample'] = f
        df = pd.concat([df_new, df])

    # If the output file exists, overwrite it
    if os.path.exists(output):
        os.remove(output)
        print("File exists, overwriting")

    # Write the dataframe to the desired location
    df.to_csv(output, header=True, index=False, sep='\t', mode='a')
    return 0


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-i', '--input', nargs='+', help='List of input files')
    parser.add_argument('-o', '--output', help='Output file name')
    args = parser.parse_args()

    amr_concat(args.input, args.output)



#!/usr/bin/env python
# Copyright 2017 Marco Galardini and John Lees

'''Summarise k-mer annotation at the gene level'''


def get_options():
    import argparse

    description = 'Summarise k-mer annotation at the gene level'
    parser = argparse.ArgumentParser(description=description)

    parser.add_argument('annotations',
                        help='Annotated k-mer file from annotate_hits.py')

    parser.add_argument('--nearby',
                        action='store_true',
                        help='Use up/downstream annotation, if not in a gene')
    parser.add_argument('--unadj-p',
                        action='store_true',
                        help='Use the unadjusted p-value (set if adjusted p-value not available)')

    return parser.parse_args()


def update_summary(summary, gene, pval, af, beta):
    if summary[gene] != {}:
        summary[gene]['count'] += 1
        summary[gene]['af'] += af
        summary[gene]['beta'] += beta
        if log10p > summary[gene]['maxp']:
            summary[gene]['maxp'] = log10p
    else:
        summary[gene]['count'] = 1
        summary[gene]['af'] = af
        summary[gene]['beta'] = beta
        summary[gene]['maxp'] = log10p


if __name__ == "__main__":
    options = get_options()

    import sys
    import collections
    from math import log10

    summary = collections.defaultdict(dict)
    with open(options.annotations, 'r') as anot_file:
        for line in anot_file:
            anot_fields = line.rstrip().split("\t")
            af = float(anot_fields[1])
            if options.unadj_p:
                pvalue = float(anot_fields[2])
            elif anot_fields[8] == "":
                sys.stderr.write(
                    "No adjusted p-value found; try with --unadj-p\n")
            else:
                pvalue = float(anot_fields[8])
            beta = abs(float(anot_fields[4]))
            # double check that there are actual hits here
            if anot_fields[-1].count(';') == 0:
                sys.stderr.write('K-mer %s seemingly has no '
                                 'annotations. Skipping\n' % anot_fields[0])
                continue
            #
            annotations = anot_fields[-1].split(",")

            if pvalue > 0:
                log10p = -log10(pvalue)
                for annotation in annotations:
                    (position, down, inside, up) = annotation.split(";")
                    if inside != "":
                        update_summary(summary, inside, log10p, af, beta)
                    elif options.nearby:
                        if down != "":
                            update_summary(summary, down, log10p, af, beta)
                        if up != "":
                            update_summary(summary, up, log10p, af, beta)

    # write output
    print("\t".join(["gene", "hits", "maxp", "avg_af", "avg_maf", "avg_beta"]))
    for gene in summary:
        af = summary[gene]['af']/summary[gene]['count']
        if af > 0.5:
            maf = 1 - af
        else:
            maf = af

        print("\t".join([gene,
                         str(summary[gene]['count']),
                         str(summary[gene]['maxp']),
                         str(af),
                         str(maf),
                         str(summary[gene]['beta']/summary[gene]['count'])]))



# ----------------------------------------------------------------------------
# Copyright (c) 2022--, Panpiper development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------
import pandas as pd
import re
import sys
from pathlib import Path

def concatenate_checkm_files(out_pref: str, input_files: list):
    """
    Concatenate content of multiple CheckM output files into a single file.

    Args:
        out_pref (str): Output prefix.
        input_files (list): List of input file paths.

    Returns:
        None
    """
    output = f"{out_pref}/Quality/CheckM/checkm_cat.txt"

    Path(output).touch()

    # Open the output file for writing
    with open(output, "w") as outfile:
        # Iterate through input files starting from the second argument
        for input_file in input_files:
            with open(input_file, "r") as infile:
                # Read and skip the header if not the first file
                if input_file != input_files[0]:
                    next(infile)
                # Read and write the content of each file to the output
                for line in infile:
                    outfile.write(line)

if __name__ == "__main__":
    # Get output prefix and input file paths from command-line arguments
    out_pref = sys.argv[1]
    input_files = sys.argv[2:]

    # Call the function
    concatenate_checkm_files(out_pref, input_files)



# ----------------------------------------------------------------------------
# Purpose:
# Process CheckM log files to extract the Completeness and Contamination
# values and save them to a tab-separated file.
# ----------------------------------------------------------------------------
# Usage:
# python checkm-log.py <checkm_log_file> <output_prefix>
# ----------------------------------------------------------------------------
# Arguments:
# checkm_log_file: Path to the CheckM log file.
# output_prefix: Path to the output prefix.
# ----------------------------------------------------------------------------
# Created by: Renee Oles
# ----------------------------------------------------------------------------
# Copyright (c) 2022--, Panpiper development team.
#
# Distributed under the terms of the Modified BSD License.
#
# The full license is in the file COPYING.txt, distributed with this software.
# ----------------------------------------------------------------------------
import pandas as pd
import re
import sys
from pathlib import Path

def touch_file(file_path):
    """
    Create an empty file if it doesn't exist.

    Args:
        file_path (str): Path to the file.

    Returns:
        None
    """
    Path(file_path).touch()

def process_file(input_file, output_file):
    """
    Process a CheckM log file and write the results to an output file.

    Args:
        input_file (str): Path to the input file.
        output_file (str): Path to the output file.

    Returns:
        None
    """
    try:
        with open(input_file, 'r') as file:
            for line in file:
                if re.search("---", line):
                    while True:
                        line = file.readline().strip()
                        if not line:
                            break
                        if not re.search("Bin", line) and not re.search("---", line) and not re.search("\\[", line):
                            input_file = input_file.replace("*Quality/Assembly_filter/", "")
                            input_file = input_file.replace("/lineage.log", "")
                            line = line.replace("filter", input_file)
                            with open(output_file, 'a') as out_file:
                                out_file.write(line + "\n")
    except FileNotFoundError:
        # If the file doesn't exist, skip the normal part and fill the line with NAs
        line = "\t".join(["NA"] * 5) + "\n"
        with open(output_file, 'a') as out_file:
            out_file.write(line)

def load_data_and_process(output_file):
    """
    Load data from a processed file and perform additional processing.

    Args:
        output_file (str): Path to the output file.

    Returns:
        pd.DataFrame: Processed DataFrame.
    """
    df_c = pd.DataFrame()
    with open(output_file, 'r') as file:
        df_c = pd.read_table(file, header=None, sep="\t")

    df_c.rename(columns={0: "first"}, inplace=True)
    df_c = df_c["first"].str.split("\s{2,}", expand=True)
    df_c = df_c[[0, 1, 11, 12, 13]]
    df_c.columns = ["Bin_ID", "Marker_lineage", "Completeness", "Contamination", "Strain_heterogeneity"]

    df_c["Contamination"] = pd.to_numeric(df_c["Contamination"])
    df_c["Completeness"] = pd.to_numeric(df_c["Completeness"])
    df_c["Strain_heterogeneity"] = pd.to_numeric(df_c["Strain_heterogeneity"])
    df_c.loc[(df_c["Strain_heterogeneity"] == 100) & (df_c["Contamination"] == 100), "Strain_heterogeneity"] = 0

    return df_c

def filter_and_save_results(df, out_pref):
    """
    Filter DataFrame and save results to a file.

    Args:
        df (pd.DataFrame): Input DataFrame.
        out_pref (str): Output prefix.

    Returns:
        None
    """
    filtered_df = df[(df["Contamination"] < 5) & (df["Completeness"] > 95)]
    filtered_df.iloc[:, 0].to_csv(f"{out_pref}/Quality/CheckM/checkm_log_filter.txt", sep="\t", index=False, header=False)

def main():
    args = sys.argv[1:]
    out_pref = args[-1]
    output_file = f"{out_pref}/Quality/CheckM/checkm_log.txt"
    args = args[:-1]

    touch_file(output_file)

    for file_path in args:
        process_file(file_path, output_file)

    processed_df = load_data_and_process(output_file)
    filter_and_save_results(processed_df, out_pref)

if __name__ == "__main__":
    main()
